/*
 * FileBrowser.java
 *
 * Copyright (c) David Pollack 2009
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package ScrobbleBerry;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import net.rim.device.api.ui.Keypad;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.util.*;

class FileBrowser extends MainScreen {
    ObjectListField list = new ObjectListField(ObjectListField.ELLIPSIS) {
        protected boolean navigationClick(int status, int time) {
            doSelect();
            return true;
        }
    };
    Vector fileList = new Vector();
    ScrobblerSession _session = null;
    String currExistingPath = "";
    
    FileBrowser(ScrobblerSession session) {
        _session = session;
        
        LabelField _title = new LabelField("ScrobbleBerry - File Browser",
                LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(_title);
        
        try {
            Enumeration rootEnum = FileSystemRegistry.listRoots();
            reRead(rootEnum, currExistingPath);
        } catch (IOException e) {
            Dialog.alert("Could not find roots.");
        }
        
        add(list);
    }
    
    public boolean keyChar(char c, int status, int time) {
        switch (c) {
            case Keypad.KEY_ENTER:
                doSelect();
                return true;
            case Keypad.KEY_ESCAPE:
                return goUp();
            default:
                return super.keyChar(c, status, time);
        }
    }
    
    void doSelect() {
        try {
            int selectedIndex = list.getSelectedIndex();
            String path = currExistingPath +
                (String)list.get(list, selectedIndex);
            FileConnection selectConn =
                (FileConnection) Connector.open("file:///" + path);
            if (selectConn.isDirectory()) {
                reRead(selectConn.list(), path);
            } else {
                String playList[] =
                    new String[fileList.size() - selectedIndex];
                for (int i = 0; i < playList.length; i++) {
                    playList[i] = "file:///" + currExistingPath +
                        (String)list.get(list, i + selectedIndex);
                }
                UiApplication.getUiApplication().pushScreen(
                                    new PlayerScreen(playList, _session));
            }
        } catch (IOException e) {
            Dialog.alert("Could not read file system.");
        }
    }
    
    void reRead(Enumeration fileListEnum, String existingPath)
            throws IOException {
        currExistingPath = existingPath;
        
        for (int i = 0; i < fileList.size(); i++) {
            list.delete(0);
        }
        fileList.removeAllElements();
        
        Vector fileListShow = new Vector();
        while (fileListEnum.hasMoreElements()) {
            String curr = (String) fileListEnum.nextElement();
            fileList.addElement(existingPath + curr);
            fileListShow.addElement(curr);
        }
        
        String[] fileListArray = new String[fileListShow.size()];
        for (int i = 0; i < fileList.size(); i++) {
            fileListArray[i] = (String) fileListShow.elementAt(i);
        }
        
        StringComparator c = StringComparator.getInstance(false);
        Arrays.sort(fileListArray, c);
        
        list.set(fileListArray);
    }
    
    boolean goUp() {
        String path = currExistingPath;
        
        if (path.indexOf('/') == -1) {
            return false;
        } else {
            try {
                Enumeration upEnum = null;
                String newPath = path.substring(0,
                    path.lastIndexOf('/', path.lastIndexOf('/')-1) + 1);
                if (newPath.length() == 0) {
                    upEnum = FileSystemRegistry.listRoots();
                } else {
                    FileConnection upConn =
                        (FileConnection) Connector.open("file:///" + newPath);
                    upEnum = upConn.list();
                }
                reRead(upEnum, newPath);
            } catch (IOException e) {
                Dialog.alert("Could not read file system.");
            }
            return true;
        }
    }
}
