/*
 * ScrobbleBerry.java
 *
 * Copyright (c) David Pollack 2009
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package ScrobbleBerry;

import java.io.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;
import javax.microedition.media.MediaException;
import net.rim.device.api.io.*;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.system.*;


class ScrobbleBerry extends UiApplication {
    static String _username = "";
    static String _password = "";
    static int _connectionIndex = 2;
    
    public static void main(String args[]) {
        ScrobbleBerry mainApp = new ScrobbleBerry();
        mainApp.enterEventDispatcher();
    }
    
    ScrobbleBerry() {
        pushScreen(new MenuScreen());
    }
    
    private static class MenuScreen extends MainScreen {
        public static final String LOGIN_RECORDSTORE_NAME = "loginStore";
        
        BasicEditField _usernameField;
        PasswordEditField _passwordField;
        ObjectChoiceField _connectionField;
        
        String _connectionNames[] = { "BES", "TCP/IP", "WiFi" };
        int _connectionConsts[] = { httpTools.BES,
                                    httpTools.TCPIP,
                                    httpTools.WIFI };
        
        ScrobblerSession currSession = new ScrobblerSession(_username,
                                _password,
                                _connectionConsts[_connectionIndex]);
        
        MenuScreen() {
            super();
            
            RecordStore rs = null;
            try {
                rs = RecordStore.openRecordStore(LOGIN_RECORDSTORE_NAME, true);
            } catch (RecordStoreException e) {
                Dialog.alert("Could not create record store.");
            }
            
            try {
                if (rs.getNextRecordID() != 1) {
                    byte[] loginData = rs.getRecord(1);
                    ByteArrayInputStream bin =
                        new ByteArrayInputStream(loginData);
                    DataInputStream din = new DataInputStream(bin);
                    
                    _username = din.readUTF();
                    _password = din.readUTF();
                    _connectionIndex = din.readInt();
                    
                    din.close();
                }
            } catch (RecordStoreException e) {
                Dialog.alert("Could not read login details.");
            } catch (IOException e) {
                Dialog.alert("Could not read login details.");
            }
            
            
            LabelField _title = new LabelField("ScrobbleBerry Settings",
                LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
            setTitle(_title);
            
            _usernameField = new BasicEditField("Last.fm username: ",
                                                _username);
            _passwordField = new PasswordEditField("Last.fm password: ",
                                                    _password);
            _connectionField = new ObjectChoiceField("Connection method: ",
                                                    _connectionNames,
                                                    _connectionIndex);
            _usernameField.setText(_username);
            _passwordField.setText(_password);
            
            add(_usernameField);
            add(_passwordField);
            add(_connectionField);
            
            MenuItem _saveConfigItem = new MenuItem("Save", 1, 1) {
                public void run() {
                    save();
                }
            };
            MenuItem _saveConnectItem = new MenuItem("Save and Connect", 1, 1) {
                public void run() {
                    save();
                    
                    Status.show("Handshaking...");

                    currSession.doHandshake();
                    
                    if (currSession.isConnected()) {
                        Dialog.alert("Connected!");
                    } else {
                        Dialog.alert("Handshake failed.");
                    }
                }
            };
            MenuItem _fileItem = new MenuItem("File Browser", 1, 1) {
                public void run() {
                    if (currSession.isConnected()) {
                        UiApplication.getUiApplication().pushScreen(
                                        new FileBrowser(currSession));
                    } else {
                        Dialog.alert("Please save and connect first.");
                    }
                }
            };
            
            addMenuItem(_saveConfigItem);
            addMenuItem(_saveConnectItem);
            addMenuItem(_fileItem);
        }
        
        public void save() {
            _username = _usernameField.getText();
            _password = _passwordField.getText();
            _connectionIndex = _connectionField.getSelectedIndex();
            
            RecordStore rs = null;
            
            try {
                rs = RecordStore.openRecordStore(LOGIN_RECORDSTORE_NAME, true);
            } catch (RecordStoreException e) {
                Dialog.alert("Could not create record store.");
            }
            
            try {
                ByteArrayOutputStream bout = new ByteArrayOutputStream();
                DataOutputStream dout = new DataOutputStream(bout);
                dout.writeUTF(_username);
                dout.writeUTF(_password);
                dout.writeInt(_connectionIndex);
                dout.close();
                byte[] loginData = bout.toByteArray();
                
                if (rs.getNextRecordID() == 1) {
                    rs.addRecord(loginData, 0, loginData.length);
                } else {
                    rs.setRecord(1, loginData, 0, loginData.length);
                }
            } catch (RecordStoreException e) {
                Dialog.alert("Could not save details.");
            } catch (IOException e) {
                Dialog.alert("Could not save details.");
            }
            
            try {
                rs.closeRecordStore();
            } catch (RecordStoreException e) {
            }
            
            currSession.setUsername(_username);
            currSession.setPassword(_password);
            currSession.setConnectionType(_connectionConsts[_connectionIndex]);
        }
    }
}
