/*
 * ScrobblerSession.java
 * 
 * Copyright (c) David Pollack 2009
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package ScrobbleBerry;

import java.io.*;
import java.lang.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;
import net.rim.device.api.io.*;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.Status;

// Implements Last.fm Submissions Protocol v1.2.1
class ScrobblerSession {
    public static final String CLIENT_ID = "bry";
    public static final String CLIENT_VERSION = "0.1";
    public static final String PROTOCOL_VERSION = "1.2.1";
    public static final String CACHE_STORE_NAME = "cacheStore";
    
    private boolean _handshaked = false;
    
    private String _username;
    private String _password;
    private int _connectionType;
    private String _sessionID;
    private String _np_url;
    private String _protocol_url;
    
    private Vector _scrobbleCache = new Vector();
    
    String getUsername() {
        return _username;
    }
    String getPassword() {
        return _password;
    }
    int getConnectionType() {
        return _connectionType;
    }
    void setUsername(String username) {
        _username = username;
    }
    void setPassword(String password) {
        _password = password;
    }
    void setConnectionType(int connectionType) {
        _connectionType = connectionType;
    }
    boolean isConnected() {
        return _handshaked;
    }
    
    ScrobblerSession(String username, String password, int connectionType) {
        _username = username;
        _password = password;
        _connectionType = connectionType;
        
        RecordStore rs = null;
        try {
            rs = RecordStore.openRecordStore(CACHE_STORE_NAME, true);
        } catch (RecordStoreException e) {
            Dialog.alert("Could not create cache store.");
        }
        int last, numRec;
        try {
            last = rs.getNextRecordID();
            numRec = rs.getNumRecords();
        } catch (RecordStoreNotOpenException e) {
            last = 1;
            numRec = 0;
        } catch (RecordStoreException e) {
            last = 1;
            numRec = 0;
        }
        int countRec = 0;
        for (int id = 1; id < last && countRec < numRec; id++) {
            try {
                byte[] scrobbleData = rs.getRecord(id);
                Scrobble loadScrobble = new Scrobble(scrobbleData);
                _scrobbleCache.addElement(loadScrobble);
                countRec++;
            } catch (InvalidRecordIDException e) {
            } catch (RecordStoreException e) {
                Dialog.alert("Could not load scrobble cache.");
            }
        }
        
        try {
            rs.closeRecordStore();
        } catch (RecordStoreException e) {
        }
    }
    
    void doHandshake() {
        long timestamp = System.currentTimeMillis() / 1000;
        String authToken =
                new MD5(new MD5(_password).asHex() + timestamp).asHex();
        
        String url = "http://post.audioscrobbler.com/?hs=true" +
                    "&p=" + PROTOCOL_VERSION +
                    "&c=" + CLIENT_ID +
                    "&v=" + CLIENT_VERSION +
                    "&u=" + URLEncode(_username) +
                    "&t=" + timestamp +
                    "&a=" + authToken;
        
        HttpConnection theConn;
        theConn = httpTools.createConn(url, null, _connectionType);
        try {
            if (theConn.getResponseCode() == HttpConnection.HTTP_OK) {
                String line;
                LineReader reader = new LineReader(
                                theConn.openInputStream());
                if ((line = new String(reader.readLine())) != null) {
                    if (line.equals("OK")) {
                        _sessionID = new String(reader.readLine());
                        _np_url = new String(reader.readLine());
                        _protocol_url = new String(reader.readLine());
                        _handshaked = true;
                    } else if (line.equals("BANNED")) {
                        Dialog.alert(
                            "This client has been banned. " +
                            "Consider upgrading?");
                    } else if (line.equals("BADAUTH")) {
                        Dialog.alert("Username or password incorrect.");
                    } else if (line.equals("BADTIME")) {
                        Dialog.alert(
"Your BlackBerry clock is too far off Last.fm server time. Please make sure " +
"the timezone is set correctly and that the time (and date) are correct.");
                    } else if (line.startsWith("FAILED")) {
                        Dialog.alert("Last.fm server failure, reason: " +
                                line);
                    } else {
                        Dialog.alert(
                                "An unknown response was received from " +
                                "the server.");
                    }
                } else {
                    Dialog.alert("HTTP OK, but no response from server.");
                }
            } else {
                Dialog.alert("Handshake fail, HTTP code "
                        + theConn.getResponseCode());
            }
            
            theConn.close();
        } catch (IOException e) {
        } catch (NullPointerException e) {
        }
    }
    
    private static String URLEncode(String field) {
        StringBuffer workingField = new StringBuffer();
        byte[] fieldBytes;
        try {
            fieldBytes = field.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            fieldBytes = field.getBytes();
        }
        for (int i = 0; i < fieldBytes.length; i++) {
            char currCh = (char) fieldBytes[i];
            if (currCh == ' ') {
                workingField.append('+');
            } else if ((currCh >= 'A' && currCh <= 'Z') ||
                        (currCh >= 'a' && currCh <= 'z') ||
                        (currCh >= '0' && currCh <= '9') ||
                        currCh == '-' || currCh == '.' ||
                        currCh == '_' || currCh == '~') {
                workingField.append(currCh);
            } else {
                workingField.append('%');
                workingField.append(Integer.toHexString(currCh).toUpperCase());
            }
        }
        
        return workingField.toString();
    }
    
    void sendNowPlaying(String artistname, String track, String album,
                        String secs, String tracknumber) {
        String response = "BADSESSION";
        while (response.equals("BADSESSION")) {
            String npPostData = "s=" + _sessionID +
                            "&a=" + URLEncode(artistname) +
                            "&t=" + URLEncode(track) +
                            "&b=" + URLEncode(album) +
                            "&l=" + URLEncode(secs) +
                            "&n=" + URLEncode(tracknumber) +
                            "&m=";

            HttpConnection npConn;
            npConn = httpTools.createConn(_np_url,
                        npPostData.getBytes(), _connectionType);
            try {
                if (npConn.getResponseCode() == HttpConnection.HTTP_OK) {
                    LineReader reader = new LineReader(
                                    npConn.openInputStream());
                    if ((response = new String(reader.readLine())) != null) {
                        if (response.equals("OK")) {
                        } else if (response.equals("BADSESSION")) {
                            doHandshake();
                        } else {
                            Dialog.alert("Now Playing error: " +
                                "unknown response received from server.");
                        }
                    } else {
                        Dialog.alert("Now Playing error: HTTP OK, " +
                            "but no response from server.");
                    }
                } else {
                    Dialog.alert("HTTP error, code " +
                            npConn.getResponseCode());
                }
                
                npConn.close();
            } catch (IOException e) {
                response = "error";
            } catch (NullPointerException e) {
                response = "error";
            }
        }
    }
    
    void addToCache(Scrobble scrobble) {
        if (!_scrobbleCache.contains(scrobble)) {
            _scrobbleCache.addElement(scrobble);
        }
    }
    
    void submitCache() {
        String resp = "OK";
        while (!_scrobbleCache.isEmpty() && resp.equals("OK")) {
            StringBuffer postData = new StringBuffer("s=" + _sessionID);
            for (int i = 0; i < _scrobbleCache.size() && i < 50; i++) {
                Scrobble currScrobble = (Scrobble) _scrobbleCache.elementAt(i);
                
                postData.append("&a[" + i + "]=" +
                                URLEncode(currScrobble.getArtist()));
                postData.append("&t[" + i + "]=" +
                                URLEncode(currScrobble.getTrack()));
                postData.append("&i[" + i + "]=" +
                                currScrobble.getTimestamp());
                postData.append("&o[" + i + "]=P");
                postData.append("&r[" + i + "]=");
                postData.append("&l[" + i + "]=" +
                                URLEncode(currScrobble.getLength()));
                postData.append("&b[" + i + "]=" +
                                URLEncode(currScrobble.getAlbum()));
                postData.append("&n[" + i + "]=" +
                                URLEncode(currScrobble.getTrackNumber()));
                postData.append("&m[" + i + "]=");
            }
            
            resp = "BADSESSION";
            while (resp.equals("BADSESSION")) {
    
                HttpConnection conn;
                conn = httpTools.createConn(_protocol_url,
                            postData.toString().getBytes(), _connectionType);
                try {
                    if (conn.getResponseCode() == HttpConnection.HTTP_OK) {
                        LineReader reader = new LineReader(
                                        conn.openInputStream());
                        if ((resp = new String(reader.readLine())) != null) {
                            if (resp.equals("OK")) {
                                if (_scrobbleCache.size() > 50) {
                                    for (int j = 0; j < 50; j++) {
                                        _scrobbleCache.removeElementAt(0);
                                    }
                                } else {
                                    _scrobbleCache.removeAllElements();
                                }
                            } else if (resp.equals("BADSESSION")) {
                                doHandshake();
                            } else if (resp.startsWith("FAILED")) {
                                Dialog.alert("Last.fm server failure, reason: "
                                + resp);
                            } else {
                                Dialog.alert("Submission error: " +
                                    "unknown response received from server.");
                            }
                        } else {
                            Dialog.alert("Submission error: HTTP OK, " +
                                "but no response from server.");
                        }
                    } else {
                        Dialog.alert("HTTP error, code " +
                                conn.getResponseCode());
                    }
                    
                    conn.close();
                } catch (IOException e) {
                    resp = "error";
                } catch (NullPointerException e) {
                    resp = "error";
                }
            }
        }
    }
    
    void sendScrobble(String artist, String track, String album,
                        String length, String tracknumber, long timestamp) {
        addToCache(new Scrobble(artist, track, album,
                                length, tracknumber, timestamp));
        submitCache();
        updateStore();
    }
    
    private void updateStore() {
        try {
            RecordStore.deleteRecordStore(CACHE_STORE_NAME);
        } catch (RecordStoreNotFoundException e) {
        } catch (RecordStoreException e) {
            Dialog.alert("Scrobble cache in use.");
        }
        
        RecordStore rs = null;
        
        try {
            rs = RecordStore.openRecordStore(CACHE_STORE_NAME, true);
        } catch (RecordStoreException e) {
            Dialog.alert("Could not create cache store.");
        }
        
        for (int i = 0; i < _scrobbleCache.size(); i++) {
            try {
                Scrobble currScrobble = (Scrobble) _scrobbleCache.elementAt(i);
                byte[] scrobbleData = currScrobble.toByteArray();
                rs.addRecord(scrobbleData, 0, scrobbleData.length);
            } catch (RecordStoreException e) {
                Status.show("Could not update cache store.");
            } catch (IOException e) {
            }
        }
        
        try {
            rs.closeRecordStore();
        } catch (RecordStoreException e) {
        }
    }
    
    private static class Scrobble {
        private String _artist;
        private String _track;
        private String _album;
        private String _length;
        private String _tracknumber;
        private long _timestamp;
        
        Scrobble(String artist, String track, String album,
                    String length, String tracknumber, long timestamp) {
            _artist = artist;
            _track = track;
            _album = album;
            _length = length;
            _tracknumber = tracknumber;
            _timestamp = timestamp;
        }
        Scrobble(byte[] scrobble) {
            try {
                fromByteArray(scrobble);
            } catch (IOException e) {
            }
        }
        
        String getArtist() {
            return _artist;
        }
        String getTrack() {
            return _track;
        }
        String getAlbum() {
            return _album;
        }
        String getLength() {
            return _length;
        }
        String getTrackNumber() {
            return _tracknumber;
        }
        long getTimestamp() {
            return _timestamp;
        }
        
        public void fromByteArray(byte[] scrobble) throws IOException {
            ByteArrayInputStream bin = new ByteArrayInputStream(scrobble);
            DataInputStream din = new DataInputStream(bin);
            
            _artist = din.readUTF();
            _track = din.readUTF();
            _album = din.readUTF();
            _length = din.readUTF();
            _tracknumber = din.readUTF();
            _timestamp = din.readLong();
            
            din.close();
        }
        
        public byte[] toByteArray() throws IOException {
            ByteArrayOutputStream bout = new ByteArrayOutputStream();
            DataOutputStream dout = new DataOutputStream(bout);
            
            dout.writeUTF(getArtist());
            dout.writeUTF(getTrack());
            dout.writeUTF(getAlbum());
            dout.writeUTF(getLength());
            dout.writeUTF(getTrackNumber());
            dout.writeLong(getTimestamp());
            
            dout.close();
            
            return bout.toByteArray();
        }
    }
}


